use axum::{body::Body, response::Response, routing::get, Router};
use sqlx::SqlitePool;
use std::sync::Arc;
use tokio::net::TcpListener;
use tower_sessions::{MemoryStore, SessionManagerLayer};

mod partials;
mod station;

const PICO_CSS: &str = include_str!("../assets/pico.classless.min.css");
const MY_CSS: &str = include_str!("../assets/style.css");

async fn serve_pico_css() -> Response<Body> {
    Response::builder()
        .header("Content-Type", "text/css")
        .body(Body::from(PICO_CSS))
        .unwrap()
}

async fn serve_my_css() -> Response<Body> {
    Response::builder()
        .header("Content-Type", "text/css")
        .body(Body::from(MY_CSS))
        .unwrap()
}

/// Starts the main application.
pub async fn start(listener: TcpListener, db: SqlitePool) {
    let session_store = MemoryStore::default();
    let session_layer = SessionManagerLayer::new(session_store);

    let app = Router::new()
        .nest("/station", station::routes())
        .route("/pico.css", get(serve_pico_css))
        .route("/style.css", get(serve_my_css))
        .with_state(Arc::new(db))
        .layer(session_layer);

    axum::serve(listener, app).await.unwrap();
}
